#include "mex.h"
#include <math.h>

// To be used as
//  [supp, num_gruop, opt_obj] = CUR_PO_recover(s, graph)
// Input: 
//   s: the weight vector on the nodes
//   graph: the sparse matrix (#var * #group) that encodes group structure
// Output:
//   supp: the support of the result of polar oracle
//   num_gruop: the number of active group of the polar result
//   opt_obj: the optimal objective value of polar oracle found by recovery (optional)
void mexFunction (int nlhs, mxArray *plhs[], 
                  int nrhs, const mxArray *prhs[]) 
{
  mwIndex num_group, num_var;
  mwIndex *jc, *ir, *ir_ori;
  mwIndex j, k, nnz, num_act_var, opt_num_var, opt_num_group;  
  double opt_obj, cur_obj, si, *s, sums, *p_supp;
  int opt_i, i, *y_v, num_node_cached, *node_cached;

  s = mxGetPr(prhs[0]);
  jc = mxGetJc(prhs[1]);    ir = mxGetIr(prhs[1]);    ir_ori = ir;

  num_var = mxGetM(prhs[1]);
  num_group = mxGetN(prhs[1]);

  node_cached = new int [num_var];
  y_v = new int [num_var];
  sums = 0;
  for (i = 0; i < num_var; i++)  {
    y_v[i] = 1;
    sums += s[i];
  }

  opt_i = -1;   opt_obj = sums / num_group;   opt_num_var = num_var;
  num_act_var = num_var;
  num_node_cached = 0;

  // y_v: 1 or 0: selected, -1, not selected; 
  // start from all groups and nodes are selected, then greedily switch off groups.
  for (i = 0; (i < num_group - 1) && (num_act_var > 0); i++)  
  {    
    nnz = jc[i+1] - jc[i];
    for (j = 0; j < nnz; j ++)
    {
      k = *(ir++);
      // switch off all nodes that are in the current group
      if (y_v[k] > 0)      
      {
        y_v[k] = 0; sums -= s[k]; num_act_var--; // 0 for probably not active
        node_cached[num_node_cached++] = k;
      }
    }

    cur_obj = sums / (num_group - 1.0 - i);
    if (cur_obj > opt_obj)    
    {
      opt_i = i;
      opt_obj = cur_obj;
      opt_num_var = num_act_var;
      while(num_node_cached > 0)
        y_v[node_cached[--num_node_cached]] = -1; // -1 for de-selected
    }
  }

  opt_num_group = 0;
  ir = ir_ori;
  for (i = 0; i < num_group; i++)  
  {    
    nnz = jc[i+1] - jc[i];

    for (j = 0; j < nnz; j ++)
    {
      if (y_v[*(ir+j)] > -1)
        break;
    }
    ir += nnz;
    if (j < nnz)
      opt_num_group ++;
  }

//  mexPrintf("opt_obj = %g, opt_num_var = %d, g1 = %d, g2 = %d\n", opt_obj, opt_num_var, 
//          num_group - 1 - opt_i, opt_num_group);

  plhs[0] = mxCreateDoubleMatrix(opt_num_var, 1, mxREAL);
//  plhs[1] = mxCreateDoubleScalar(opt_num_group);
  plhs[1] = mxCreateDoubleScalar(num_group - 1.0 - opt_i);

  if (nlhs > 2)
    plhs[2] = mxCreateDoubleScalar(opt_obj);

  p_supp = mxGetPr(plhs[0]);
  for (i = 0; i < num_var; i ++)
  {
    if (y_v[i] > -1)
      *(p_supp++) = i+1;
  }

  delete [] y_v;    delete [] node_cached;
}
