%  [X, fval] = prox_op_group(G, prox_par, par): given G, solves 
%     (*)  argmin_X <X, -G> + lambda * reg(X) + 1/2 ||X||^2
%       
%     This is equivalent to 
%     (**) min_X 1/2 ||X - G||^2 + lambda * reg(X)
%  Input:
%     G: the gradient
%     prox_par: parameters passed from Nesteov solver
%     par: parameters for the SPAMS routine
%  Output:
%     X: the proximal projection
%     fval: the value of the proximal objective:
%       fval.reg: value of the regularizer at the solution X (not multiplied by lambda)
%       fval.obj_lin: value of the proximal operator objective (*)
%       fval.obj_MY: value of the proximal operator objective (**) as Moreau-Yosida regularizer

function [X, fval] = prox_op_group(G, prox_par, par)

  lambda = prox_par.weight_reg;
  par.par_prox.lambda = lambda;
  
  if isinf(par.exp_par.p)    
    [X, fval.reg] = mexProximalGraph(G(:), par.graph, par.par_prox);
    fval.obj_lin = lambda*fval.reg + sum(sum(X.*(X-2*G)))/2;
    fval.obj_MY = lambda*fval.reg + norm(X - G, 'fro')/2;
    
% double check (give up as the algorithm needs a huge recursion level)
%     sbd_ctx.G_by_V = par.graph.G_by_V;
%     sbd_ctx.V_by_G = par.graph.V_by_G;
%     [X1, fval1] = prox_op_sbd(G(:), lambda, sbd_ctx, par);
%     fprintf('mex: %g, %g, %g\n', fval.reg, fval.obj_lin, fval.obj_MY);
%     fprintf('mine: %g, %g, %g\n', fval1.reg, fval1.obj_lin, fval1.obj_MY);
  else
    % use the fancy recursion
    [X, fval] = prox_op_sbd(G(:), lambda, par.sbd_ctx, par);
  end
  
  if ~isvec(G)
    X = reshape(X, size(G));
  end
  
end


