% [v, supp, fA, opt_obj] = polar_op_Secant(g, A0, opt)
% Input:
%   g: weights on the nodes
%   A0: initial guess of the optimal A
%   par: parameters. Eg, par.graph.G_by_V gives the graph.
% Output:
%  v = pola(g) with 0's in the complement of C(g) ignored
%  supp = C(g), a 0/1 vector representing the optimal set A
%  fA = F(C(g))^{1/q}
%  opt_obj: the optimal value of <g.^q, 1_A> / F(A)

function [v, supp, fA, opt_obj] = polar_op_Secant(g, par)
  
  p = par.exp_par.p;
  if isinf(p), q = 1; else q = p/(p-1); end
  s = abs(g).^q;
  
  [A_pre, fA_pre] = par.get_secant_init(s, par.sbd_ctx, par);
  lambda = sum(s(A_pre)) / fA_pre;
  while 1
    [A, fA, opt_obj] = par.sbd_oracle(s, lambda, par.sbd_ctx, par);
    fprintf('opt_obj = %g\n', opt_obj);
    if opt_obj < par.polar_par.secant_tol
      if sum(A) == 0
        A = A_pre;  fA = fA_pre;
      end
      break;
    end
    A_pre = A;  fA_pre = fA;
    
    lambda = lambda + opt_obj / fA;
%     assert(abs(lambda - sum(s(A))/fA) < 1e-5);
  end
    
  v = g(A);
  sum_sA = sum(s(A));
  if isinf(p)
    vv = sign(v);
  else
    vv = sign(v) .* (abs(v).^(q-1)) / sum_sA^(1/p);
  end
  
  % Check by Holder's inequality
  assert(abs(v'*vv - sum_sA^(1/q)) < 1e-6);

  opt_obj = sum_sA / fA;
  fA = fA^(1/q);
  v = vv / fA;  
  supp = find(A);
%   fprintf('opt polar obj by secant = %g\n', lambda);
  
  if par.polar_par.check_LP
    ntotal = num_group + num_var;
    graph = par.graph;
    [sol, f] = linprog([-g; zeros(num_group,1)], graph.A, graph.b, ...
                graph.Aeq, graph.beq, zeros(ntotal,1), ones(ntotal,1), ...
                [zeros(num_var,1); ones(num_group,1)/num_group]);
    cprintf('red', 'LP obj = %g\n', f);
  end

end
