function [h] = recover_h(Z, d1, ub)

G = null_space(Z, 1);
rank_null = size(G, 2);
G0 = G(1, :);
G1 = G(2:1+d1,:);
G2 = G(d1+2:end,:);

if rank_null == 1
  h = G / G(1);
  fprintf('Recover h with only one basis.\n');
  return;
end

A = G1'*(G1/ub(1)^2);
B = G2'*(G2/ub(2)^2);

% co-diagonalize A and B
[P, D] = eig(A);
diag_D = diag(D);

margin = max(diag_D);
diag_Dbar_sqrt = sqrt(diag_D + margin);

[U, Lam] = laneig(@aux_eig_recover_h, rank_null);
diag_Lam = diag(Lam);

% Compute beta based on Lambda
[max_lam, idx_max] = max(diag_Lam);
[min_lam, idx_min] = min(diag_Lam);
if max_lam < 1 || min_lam > 1
%   fprintf('Cannot recover h\n');
  h = G(:, end);  h = h / h(1);
  return;
end

beta = zeros(rank_null, 1);
beta(idx_max) = sqrt((1 - min_lam)/margin);
beta(idx_min) = sqrt((max_lam - 1)/margin);

% Translate alpha to beta
alpha = U'*(P*((P'*beta) ./ diag_Dbar_sqrt));

% Test validity of alpha
if abs(G0*alpha) < 1e-4
  fprintf('Warning in recover_h: beta makes 0\n');
  keyboard;
end

h = G*alpha;    

h(2:1+d1) = h(2:1+d1)/h(1);  % * (ub(1)/h(1));
h(2+d1:end) = h(2+d1:end)/h(1); % * (ub(2)/h(1));
h(1) = 1;
nm = norm(h(2:1+d1)); if nm > ub(1), h(2:1+d1) = h(2:1+d1)*(ub(1)/nm);  end
nm = norm(h(2+d1:end)); if nm > ub(2), h(2+d1:end) = h(2+d1:end)*(ub(2)/nm);  end

  function [output] = aux_eig_recover_h(input)
    t = P*((P'*input) ./ diag_Dbar_sqrt);
    t = B*t + margin*t;
    output = P*((P'*t) ./ diag_Dbar_sqrt);
  end

end