% Use GCG on multiview dataset
%
function [sol, obj, res] = solve_LOB_mv(objf, lambda, X0, evalf, opts)
        
  % Partially corrective boosting
  % Solve  Min_X  func(X) + lambda * ||X||_mv

  [m, n] = size(X0);
  m1 = opts.prob_par.m1;  
  alpha = opts.prob_par.alpha;
  beta = opts.prob_par.beta;
  gvalue = get_global_value();
  
  k = opts.init_rank;  % Initialize U and V with rank k

  U = randn(m, k);
  V = randn(k, n);

  res = struct('obj_list', [], ...
               'loss_list', [], ...
               'perf_list', [] ...
               );
  
  msg = 'Running';
  break_flag = 0;
  perf = struct('l1', 0, 'l2', 0);  
  par = opts.ori_par;
  nidx = k;
  obj = inf;
  
  time_list = [];
  time_local = [];
  if opts.use_local
    time_polar = [];
  else
    time_polar = 0;
  end
  
  t1 = cputime;  
  for i = 1 : opts.max_iter
    if i == 25
      par.lbfgsb_in.maxIter = 60;
      par.lbfgsb_in.maxFnCall = 60;
    end

    k = size(U, 2);

    % Local Search
    if k > 0
      t2 = cputime;
      if opts.use_local
        
        nel = k*(m+n);
        ub = Inf(nel, 1);
        % local search    
        disp('local search now');
        
%         x0 = [U(:); V(:)];
%         [ff, gg] = obj_UV(x0);
%         [gtest, err] = gradest(@obj_UV, x0);
        
        
        [UV, obj, ~, ~, msg] = lbfgsb([U(:); V(:)], -ub, ub, @obj_UV_hard, [], ...
                                      opts.lbfgsb_in.view, opts.lbfgsb_in);
        U = reshape(UV(1:k*m), [m, k]);
        V = reshape(UV(1+k*m:end), [k, n]);
      end
      
      time_local = [time_local; cputime - t2];
      time_list = [time_list; cputime - t1];
      
      if isinf(obj)
        obj = obj_UV(U*V);
      end
      
      norm_U = max([sqrt(sum(U(1:m1,:).^2))/alpha; ...
                    sqrt(sum(U(1+m1:end,:).^2))/beta])';
      norm_V = sqrt(sum(V.*V, 2));
      norm_UV = sqrt(norm_U .* norm_V);

      loss = objf.obj(U*V);
      obj = loss + 0.5*lambda*(norm(norm_U)^2 + norm(norm_V)^2);
%       loss = obj - 0.5*lambda*(norm(U, 'fro')^2 + norm(V, 'fro')^2);
%       loss = obj - 0.5*lambda*(norm(norm_U)^2 + norm(norm_V)^2);
      res.loss_list = [res.loss_list; loss];    
      res.obj_list = [res.obj_list; obj];
      
      if ~isempty(evalf)
        perf = evalf(U*V);
        res.perf_list = [res.perf_list; perf];
      end

      % Prune U and V
      score = sqrt(norm_U.^2 + norm_V.^2); 
      idx = score > 1e-5;
      nidx = sum(idx);
      
      if nidx < size(U, 2)
        U = U(:, idx);
        V = V(idx, :);
        norm_U = norm_U(idx);
        norm_V = norm_V(idx);
        norm_UV = norm_UV(idx);
      end
%       reg = lambda*kappa(U, V, opts);
%       
%       gvalue.col_print('diff = %g\n', obj-loss-reg);
%       obj = loss + reg;            
      gvalue.col_print('iter=%d, loss = %g, obj=%g, perf=(%g, %g), k=%d, time=%g, ls_time=%g, msg=%s\n', ...
            i, loss, obj, perf.l1, perf.l2, k, sum(time_list), sum(time_local), msg2str(msg, 'lbfgsb'));
      
    end

    if i > 1
      if abs(pre_obj-obj) / min(abs([pre_obj, obj])) < opts.rtol
        outputs.msg = 'Stop with small relative change';
        break_flag = 1;
      elseif sum(time_list) > opts.max_time
        outputs.msg = 'Stop by max_time';
        break_flag = 1;
      elseif i == opts.max_iter
        outputs.msg = 'Stop with max iteration';  
        break_flag = 1;
      end
    end
    
    if (opts.use_local && k > 0) || i > 1 
      res.time_list = cumsum(time_list);
      res.time_polar = cumsum(time_polar);
      res.time_local = cumsum(time_local);
      res.msg = msg;
      res.iter = i;

      if isfield(opts, 'res_fname')
        save(opts.res_fname, 'res', 'par');
      end
    end
    
    if break_flag == 1, break;    end
        
    pre_obj = obj;
    t1 = cputime;
    if nidx > 0
      sk = 0.5*(norm_U'*norm_U + norm_V'*norm_V);
    else
      sk = 0;
    end

    % Get a gradient and compute the polar oracle
    if size(U, 2) > 0
      [~, G] = objf.obj(U*V);    % G is a sparse matrix
    else
      [~, G] = objf.obj(zeros(m, n)); 
    end

    % polar oracle
    t2 = cputime;
    [u, v] = opts.polar_op(G);
    time_polar = [time_polar; cputime - t2];
    clear G
    v = -v;

    % 2-D line search
    disp('2-D line search now');
    cur_UV = U*V; cur_uv = u*v';
    [weights, obj, ~, ~, msg] = lbfgsb([1; 0], [0; 0], [inf; inf], ...
                                        @obj_ls, [], [], opts.lbfgsb_in);

%     loss = obj - lambda*(sk*weights(1) + weights(2));
    weights = sqrt(weights);
    if nidx > 0
      U = [bsxfun(@times, U, weights(1)*(norm_UV./norm_U)'), weights(2) * u];
      V = [bsxfun(@times, V, weights(1)*norm_UV./norm_V); weights(2) * v']; 
    else
      U = weights(2) * u;
      V = weights(2) * v';
    end

  end

  sol = {U, V'};
  outputs.iter = i;

  if opts.verbose
    fprintf(1, '%s\n', outputs.msg);
  end
  
    % local search objective
    function [f, g] = obj_UV_hard(X)
      r = length(X) / (m+n);
      UU = reshape(X(1:r*m), [m, r]);
      VV = reshape(X(1+r*m:end), [r, n]);

      [f, GG] = objf.obj(UU*VV);
      nm1 = sum(UU(1:m1,:).^2)/alpha^2;
      nm2 = sum(UU(1+m1:end,:).^2)/beta^2;
      idx = nm1 > nm2;
      f = f + lambda/2*(sum(max([nm1; nm2])) + norm(VV, 'fro')^2);
      G1 = zeros(m, r);
      G1(1:m1, idx) = (lambda / alpha^2) * UU(1:m1,idx);
      G1(1+m1:end, ~idx) = (lambda / beta^2) * UU(1+m1:end,~idx);
      g = [vec(GG*VV' + G1)
           vec(UU'*GG + lambda*VV)];
    end

    function [f, g] = obj_UV(X)
      r = length(X) / (m+n);
      UU = reshape(X(1:r*m), [m, r]);
      VV = reshape(X(1+r*m:end), [r, n]);
      sigma = 1e-3;
      [f, GG] = objf.obj(UU*VV);
      nm1 = sum(UU(1:m1,:).^2)/alpha^2;
      nm2 = sum(UU(1+m1:end,:).^2)/beta^2;
      val = [nm1; nm2];
      maxval = max(val);
      val = val - [maxval; maxval];
      
      expval = exp(val/sigma);
      sumexpval = sum(expval);
      
      f = f + lambda/2*(sum(log(sumexpval))*sigma + sum(maxval) + norm(VV, 'fro')^2);
      
      p = expval ./ repmat(sumexpval,2,1);
      G1 = [repmat((lambda/alpha^2)*p(1,:), m1, 1) .* UU(1:m1,:)
            repmat((lambda/beta^2)*p(2,:), m-m1, 1) .* UU(1+m1:end,:)];
      g = [vec(GG*VV' + G1)
           vec(UU'*GG + lambda*VV)];
    end
  
  % Line search objective
    function [f, g] = obj_ls(x)
%       [f, GG] = objf.obj({[sqrt(x(1))*U, sqrt(x(2))*u], [sqrt(x(1))*V', sqrt(x(2))*v]});
      [f, GG] = objf.obj(x(1)*cur_UV + x(2)*cur_uv);
      f = f + lambda*(sk*x(1) + x(2));
      g = [sum(sum(GG .* cur_UV))+lambda*sk
           u'*(GG*v)+lambda];
    end
end
