function test_hybridSDP
randn('state',1)
warning off;
clc;

m = 300; n = 100;
k = 1;
lambda = 20;

A = randn(m, n);
[U, S, V] = svd(A, 'econ');
S = max(0, S-lambda);
nnz(diag(S)>1e-3)
Xstar = U*S*V';
fstar = norm(A-Xstar, 'fro')^2/2;
fprintf(1, 'optimal obj = %f\n', fstar);

B = sum(diag(S));
V = zeros(m+n, k);

obj = @(V) L2(V, A, m);

opts.max_iter = 200; opts.rtol = 1e-5;


opts.lstype = 0;
tic
[V0, f0, iter, msg, output] = hybridSDP(obj, 2*B, V, [], opts);
t0 = toc;
fprintf(1, 'no ls, obj = %f, time = %f \n', f0, t0);

opts.lstype = 1;
tic
[V1, f1, iter1, msg1, output1] = hybridSDP(obj, 2*B, V, [], opts);
t1 = toc;
fprintf(1, 'only b, obj = %f, time = %f \n', f1, t1);

opts.lstype = 2;
tic
[V2, f2, iter2, msg2, output2] = hybridSDP(obj, 2*B, V, [], opts);
t2 = toc;
fprintf(1, 'a & b, obj = %f, time = %f \n', f2, t2);

opts.lstype = -1;
tic
[V3, f3, iter3, msg3, output3] = hybridSDP(obj, 2*B, V, [], opts);
t3 = toc;
fprintf(1, 'a + b <= 1, obj = %f, time = %f \n', f3, t3);



function [f, G] = L2(V, A, m)

X = V{1}(1:m,:)*V{1}(1+m:end,:)';
f = norm(A-X, 'fro')^2/2;
G{1} = [zeros(size(X,1)) X - A
        X'-A' zeros(size(X,2))];